/* $Id: list.h,v 1.1 2000/03/13 04:51:41 ajapted Exp $
******************************************************************************

   LibGFFD - doubly linked list API

   Copyright (C) 2000  Andrew Apted    [andrew.apted@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GFFD_INTERNAL_LIST_H
#define _GFFD_INTERNAL_LIST_H


/* list structures */

typedef struct d_node
{
  struct d_node *succ;
  struct d_node *pred;
} 
d_node_t;


typedef struct d_list
{
  d_node_t *head;
  d_node_t *tail;
} 
d_list_t;


/* list operations */

void listInit(d_list_t *L);
void listFree(d_list_t *L, void (* free_func)(d_node_t *N));

d_node_t *listNewNode(int size);
void listFreeNode(d_node_t *N);

void listAddHead(d_list_t *L, d_node_t *N);
void listAddTail(d_list_t *L, d_node_t *N);
d_node_t *listRemoveNode(d_list_t *L, d_node_t *N);

void listInsertBefore(d_list_t *L, d_node_t *N, d_node_t *next);
void listInsertAfter (d_list_t *L, d_node_t *N, d_node_t *previous);

void listConcatHead(d_list_t *L, d_list_t *old);
void listConcatTail(d_list_t *L, d_list_t *old);

int listCountNodes(d_list_t *L);
d_node_t *listIndexNode(d_list_t *L, int index);
d_node_t *listFindNode(d_list_t *L, void *priv,
		      int (* search_func)(d_node_t *N, void *priv));


#endif  /* _GFFD_INTERNAL_LIST_H */
